close all
clear all
clc

% nacteni dat, zatim chceme jen produkci Y, tj. sezonne ocisteny realny
% ctvrtletni HDP CR v mld. Kc za obdobi 1995Q1-2004Q4

load data_GDPr;

n = length(Y);
t = (1:n)';


%-------------------------------------------------------------
% Odhad nepozorovane veliciny KF, vlastnosti KF
%-------------------------------------------------------------
% KF slouzi k odhadu stavu systemu, u ktereho zname podobu a parametry.
% Odhadovane stavy nemusi byt pozorovatelne. KF je rekurzivni algoritmus 
% poskytujici predikovane hodnoty, filtrovane a vyhlazne. 
% Soucasti odhadu jsou i odpovidajici kovariancni matice stavu.

%-------------------------------------------------------------
% Dekompozice realneho HDP
%-------------------------------------------------------------
% Uvazme ekonomicky priklad kdy se CB zajima o odhad hospodarskeho cyklu.
% Odhadnuta vystupova mezera je indikatorem poptavkovych inflacnich
% tlaku spise nez vyuziti kapacit. 
% Potencialni produkt je pak produkt ovlivneny vyvojem investic atd. 
% Je to v podstate "trend" v casove rade.

% V prvni nejjednodusi verzi modelu budeme predpokladat ze vystupova mezera
% je bilym sumem a potencialni produkt nahodna prochazka s tempem rustu.
% tempo rustu muzeme menit a divat se na dopady techto zmen. 
% Sumy nastavime tak, aby vysledny odhad odpovidal nasi ekonmicke intuici. 
% Podobne je tomu i v pripade pocatecnich podminek.

% Budeme pracovat s logaritmy promennych, protoze rozdil logaritmu v case
% odpovida priblizne procentni zmene.

% definice promennych
LGDP = 100*log(Y);

figure
plot(roky, LGDP)
hold on
grid
title('logaritmus realneho ceskeho HDP')

% V casove rade je trend.
% Prumerne tempo rustu je (v rocnim vyjadreni):
%dva mozne zpusoby vypoctu, pricemz rozdil ve vyslednych hodnotach je zpusoben
%tim, ze log(Y_t+1)-log(Y_t) je jen aproximace tempa
%rustu
ptr_LGDP_1=(geomean((LGDP(2:end)-LGDP(1:end-1))+100*ones(length(LGDP)-1,1))-100)*4

%mel by byt presny, protoze neni odhadnut pomoci zadne aproximace
ptr_LGDP_2=((Y(end)/Y(1))^(1/(n-1))-1)*400

% Tempo je relativne vysoke (2.21%), presto u rozvijejici se zeme by melo byt
% vyssi. Dejme alespon 3% jakozto vstup "u".

% Vratime se do ctvrtletniho vyjadreni kvuli kompatibilite s daty
% Melo by byt asi spravneji: u = ptr_LGDP*ones(size(LGDP))/4;
u = 3*ones(size(LGDP))/4;  % ctvrtletni rust: 3/4

%---------------------------------
% Model I - jednoducha verze
%---------------------------------
%
% stavove rovnice
% 1) LGDP_EQ(t)  = LGDP_EQ(t-1) + growth + v1(t) => skoro primka
% 2) LGDP_GAP(t) = v2(t) 
%
% vystupova rovnice
% LGDP(t) = LGDP_EQ(t) + LGDP_GAP(t)
%
% kde LGDP je prirozeny logaritmus GDP
% LGDP_EQ oznacuje potencilani(rovnovazny produkt)
% LGDP_GAP znaci mezeru produktu.

% Definice modelu  - stavovy popis



% >>>-----> DOPLNIT DEFINICI MODELU A ODHAD Kalmanem (filtr i smoother) a vykreslit do obrazku
























%-------------------------------------------------------------
%%% Vylepseni modelu
%-------------------------------------------------------------

% Tempo rustu bude dalsi stavovou velicinou( v predchozim pripade to byl
% vstup) a vstupem bude jen rovnovazne tempo rustu. Nase stavova velicina 
% tempa rustu potencialu - growth - bude persistentni (autoregrese) a bude 
% se blizit steady-state rustu. Ten jsme aproximovali prumernym tempem 
% vypoctenym z rady GDP.

%---------------------------------
% Model II - lepsi verze
%---------------------------------

% vystupova rovnice
% LGDP(t) = LGDP_GAP(t) + LGDP_EQ(t)
%
% stavove rovnice
% LGDP_EQ(t)  = LGDP_EQ(t-1) + growth(t) + v1(t) => muze lepe kopirovat zakriveni LGDP
% LGDP_GAP(t) = v2(t) 
% growth(t) = 0.6*growth(t-1) + 0.4*growth_ss+v3(t)


% >>>-----> DOPLNIT DEFINICI MODELU A ODHAD Kalmanem (filtr i smoother) a
% vykreslit do obrazku

















%---------------------------------
% Model III - nejlepsi verze
%---------------------------------

% V predchozich pripadech jsme predpokladali, ze gap GDP neni
% autokorelovany, coz je dost nerealisticke. Ve skutecnosti je gap GDP (tj.
% pozice v cyklu) zavisly na predchozich hodnotach (na predchozi pozici).
% Proto specifikujeme LGDP_GAP jako AR(1) proces
% vystupova rovnice
% LGDP(t) = LGDP_GAP(t) + LGDP_EQ(t)
%
% stavove rovnice
% LGDP_EQ(t)  = LGDP_EQ(t-1) + growth(t) + v1(t) => muze lepe kopirovat zakriveni LGDP
% LGDP_GAP(t) = 0.5*LGDP_GAP(t-1) + v2(t) => muze lepe zachytit persistenci cyklu
% growth(t) = 0.6*growth(t-1) + 0.4*growth_ss+v3(t)


% >>>-----> DOPLNIT DEFINICI MODELU A ODHAD Kalmanem (filtr i smoother) a
% vykreslit do obrazku





















